/*
  MatchJob.cpp - Adjust job board difficulty to match the player.

  Jason Hood, 27 April, 2012.
*/


#include "Common.h"
#include "Server.h"
#include <vector>

#define NAKED	__declspec(naked)
#define EXPORT	__declspec(dllexport)
#define STDCALL __stdcall


//#define ADDR_DIFF10 (0x703ac64+2 - 0x6f90000 + content)
#define ADDR_DIFF10 (0x6ff49bb+1 - 0x6f90000 + content)
#define ADDR_OPEN10 (0x7013fc0	 - 0x6f90000 + content)

//#define ADDR_DIFF11 (0x6f4ad64+2 - 0x6ea0000 + content)
#define ADDR_DIFF11 (0x6f046eb+1 - 0x6ea0000 + content)
#define ADDR_OPEN11 (0x6f24280	 - 0x6ea0000 + content)

// Use the header timestamp for version check.
#define ADDR_VER  ((PDWORD)(0x100 + content))
#define VER10	  0x3E401CD4
#define VER11	  0x3ECBB142


typedef bool (*TOpenIni)( LPCSTR, INI_Reader& );
TOpenIni OpenIni;


DWORD dummy;
#define ProtectX( addr, size ) \
  VirtualProtect( addr, size, PAGE_EXECUTE_READWRITE, &dummy )

#define RELOFS( from, to ) \
  *(PDWORD)(from) = (DWORD)(to) - (DWORD)(from) - 4

#define NEWOFS( from, to ) \
  to##_Old = (DWORD)(from) + *(PDWORD)(from) + 4; \
  RELOFS( from, to##_Hook )

#define INDIRECT( from, to ) \
  to##_Old = **(PDWORD*)(from); \
  to##_New = (DWORD)to##_Hook; \
  *(PDWORD*)(from) = &to##_New


struct IDLL
{
  virtual void whatever( int, LPVOID ) { }
};


struct RankDiff
{
  int	rank;
  float diff_min;
  float diff_max;
};


typedef std::vector<RankDiff> VRankDiffDB;
typedef VRankDiffDB::const_iterator VCIRankDiffDB;

VRankDiffDB RankDiffDB;

char db_file[] = "missions\\RankDiff?P.ini";


bool set_rank_diff()
{
  db_file[17] = (pub::SinglePlayer()) ? 'S' : 'M';
  INI_Reader ini;
  if (!OpenIni( db_file, ini ))
    return false;

  while (ini.read_header())
  {
    if (ini.is_header( "RankDiffDB" ))
    {
      while (ini.read_value())
      {
	if (ini.is_value( "rank_diff" ))
	{
	  RankDiff rd;
	  rd.rank     = ini.get_value_int( 0 );
	  rd.diff_min = ini.get_value_float( 1 );
	  rd.diff_max = ini.get_value_float( 2 );
	  RankDiffDB.push_back( rd );
	}
      }
    }
  }

  return !RankDiffDB.empty();
}


void STDCALL adjust_diff( UINT player, float& min_diff, float& max_diff )
{
  int rank;

  if (pub::SinglePlayer())
    pub::Player::GetRank( player, rank );
  else
  {
    int   rep;
    float frank;
    pub::Player::GetRep( player, rep );
    pub::Reputation::GetRank( rep, frank );
    rank = frank;
  }

  int i = RankDiffDB.size();
  while (--i >= 0)
  {
    if (rank >= RankDiffDB[i].rank)
    {
      if (RankDiffDB[i].diff_min > min_diff)
	min_diff = RankDiffDB[i].diff_min;
      if (RankDiffDB[i].diff_max > max_diff)
	max_diff = RankDiffDB[i].diff_max;
      break;
    }
  }
}


DWORD Diff_Old; //, Diff_New;

NAKED
void Diff_Hook()
{
  __asm {
	//mov	eax, [0x18+esp+4]
	//lea	ecx, [0x18+esp+8]
	//lea	edx, [0x18+esp+12]

	lea	ecx, [esp+8]
	lea	edx, [esp+12]
	push	edx
	push	ecx
	push	[eax+12]
	call	adjust_diff
	jmp	Diff_Old

	align	16
  }
}


extern "C" EXPORT
IDLL* CreateInstance( LPVOID, LPCSTR, LPVOID )
{
  PBYTE addr;
  PBYTE content = (PBYTE)GetModuleHandle( "content.dll" );

  switch (*ADDR_VER)
  {
    case VER11:
      addr    = ADDR_DIFF11;
      OpenIni = (TOpenIni)ADDR_OPEN11;
      break;

    case VER10:
      addr    = ADDR_DIFF10;
      OpenIni = (TOpenIni)ADDR_OPEN10;
      break;

    default:
      return NULL;
  }

  if (!set_rank_diff())
    return NULL;

  ProtectX( addr, 4 );
  //INDIRECT( addr, Diff );
  NEWOFS( addr, Diff );

  return new IDLL;
}


extern "C" EXPORT
void DestroyInstance( IDLL* )
{
  return;
}


BOOL WINAPI DllMain( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved )
{
  return TRUE;
}
